@@ -102,8 +102,8 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
 .calc_sXaug_Re_spprec <- function(locsXaug, ## conforming template
                                   X.Re) {
   distinct.X.ReML <- attr(X.Re,"distinct.X.ReML") # This fn should be called only for non-stdd REML
-  AUGIO_ZX <- as.list(locsXaug$AUGI0_ZX)
-  locX <- AUGIO_ZX$X.pv
+  AUGI0_ZX <- as.list(locsXaug$AUGI0_ZX)
+  locX <- AUGI0_ZX$X.pv
   if ( distinct.X.ReML[1L] ) {
     locX <- locX[,-(attr(X.Re,"unrestricting_cols"))]
   } 
@@ -112,8 +112,8 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
     suppl_cols <- X.Re[,extra_vars, drop=FALSE]
     locX <- cbind(locX,suppl_cols)
   }
-  AUGIO_ZX$X.pv <- locX
-  locsXaug <- def_AUGI0_ZX_sparsePrecision(AUGI0_ZX = list2env(AUGIO_ZX),
+  AUGI0_ZX$X.pv <- locX
+  locsXaug <- def_AUGI0_ZX_sparsePrecision(AUGI0_ZX = list2env(AUGI0_ZX),
                                            w.ranef=attr(locsXaug,"w.ranef"),
                                            cum_n_u_h=attr(locsXaug,"cum_n_u_h"),
                                            w.resid=attr(locsXaug,"w.resid"),
@@ -481,12 +481,13 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
   return(ZAL)
 }
 
-.make_Xscal <- function(ZAL, ZAL_scaling=NULL, AUGI0_ZX, as_matrix) {
+.make_Xscal <- function(ZAL, ZAL_scaling=NULL, processed, as_matrix) {
   if (inherits(ZAL,"ZAXlist")) ZAL <- .ad_hoc_cbind(ZAL@LIST, as_matrix=as_matrix )
   # capture programming error for ZAL_scaling:
   if (length(ZAL_scaling)==1L && ncol(ZAL)!=1L) stop("ZAL_scaling should be a full-length vector, or NULL. Contact the maintainer.")
   # ncol(ZAL)=1L could occur in 'legal' (albeit dubious) use. The total number of levels of random effects has been checked in preprocessing.
   if ( ! is.null(ZAL_scaling)) ZAL <- .m_Matrix_times_Dvec(ZAL,ZAL_scaling)
+  AUGI0_ZX <- processed$AUGI0_ZX
   if (is.null(Zero_sparseX <- AUGI0_ZX$Zero_sparseX)) Zero_sparseX <- rbind2(AUGI0_ZX$ZeroBlock, AUGI0_ZX$X.pv)
   if (inherits(ZAL,"dgCMatrix")) {
     I_ZAL <- .adhoc_rbind_I_dgC(nrow(AUGI0_ZX$I), ZAL) ## this is faster...
@@ -494,6 +495,7 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
   if (inherits(I_ZAL,"dgCMatrix") &&  inherits(Zero_sparseX,"dgCMatrix") ) {
     Xscal <- .cbind_dgC_dgC(I_ZAL, Zero_sparseX) # substantially faster than the general alternative 
   } else Xscal <- cbind2(I_ZAL, Zero_sparseX)
+  attr(Xscal,"AUGI0_ZX") <- AUGI0_ZX # environment => cheap access to its 'envir$updateable' variable or anything else 
   return(Xscal)
 }
 
@@ -513,7 +515,7 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
   )
 }
 
-.adhoc_cbind_dtC_sXaug_pxy_o <- function(sXaug, pwy_o, n_u_h) {
+.adhoc_cbind_dgC_sXaug_pxy_o <- function(sXaug, pwy_o, n_u_h) {
   I00_ZXy <- sXaug
   I00_ZXy@p <- c(I00_ZXy@p, I00_ZXy@p[length(I00_ZXy@p)]+length(pwy_o))
   I00_ZXy@i <- c(I00_ZXy@i, n_u_h-1L+seq_len(length(pwy_o))) ## fails if n_u_h is not integer
@@ -576,7 +578,7 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
 # is computed (no explicit chol facto; this is a small block). The 'r_yy' block is trivially 1*1 and it's its square 'ryy2' 
 # which is reported (rather that its trivial logdet), computed also as (essentially) a difference of crossproducts of 
 # 1-col terms. Forming these differences is however a bit clumsy.
-.get_absdiagR_blocks <- function(sXaug_blocks, pwy_o, n_u_h, processed, augZXy_solver,update_info) {
+.get_absdiagR_blocks <- function(sXaug_blocks, pwy_o, n_u_h, processed, augZXy_solver,update_info) { # for SPCORR !
   seq_n_u_h <- seq(n_u_h)
   tZW <- t(sXaug_blocks$ZW) # actually a ZL rather than a Z.
   if (is.null(template <- processed$AUGI0_ZX$template_CHM_ZZ_blocks)) { 
@@ -593,71 +595,77 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
       processed$AUGI0_ZX$template_CHM_ZZ_blocks <- CHM_ZZ
     }
   } else CHM_ZZ <- Matrix::.updateCHMfactor(template, tZW, mult=1) # no need to compute the crossprod for updating: the tcrossfac is enough.
-  XW <- sXaug_blocks$XW
-  ZtWX <- tZW %*% XW
   # perm <- as(CHM_ZZ,"pMatrix")@perm # remarkably slow... and using   perm <- CHM_ZZ@perm+1L + [perm,] is much slower than:
-  Rzx <- solve(CHM_ZZ, solve(CHM_ZZ,ZtWX,system="P"), system="L") # (maybe) dense but dge... # solve(CHM_ZZ, ZtWX[perm,], system="L") # 
   ZtWy <- tZW %*% pwy_o
   r_Zy <- solve(CHM_ZZ, solve(CHM_ZZ,ZtWy,system="P"), system="L") # solve(CHM_ZZ, ZtWy[perm], system="L")  # 
-  XtWy <- .Rcpp_crossprod(XW, pwy_o, keep_names=FALSE,as_mat=TRUE)
   #
   #cross_Rxx <- .crossprod(XW,as_mat=TRUE)-.crossprod(Rzx,as_mat=TRUE) # as(,"dpoMatrix) involves a chol() factorization...
   # Calling directly .Rcpp_crossprod avoids some bureaucratic overhead (irrespective of keep_names which could rather affect later computations)
-  cross_Rxx <- .Rcpp_crossprod(XW,BB=NULL, keep_names=FALSE,as_mat=TRUE) -
-    .Rcpp_crossprod(Rzx,BB=NULL, keep_names=FALSE,as_mat=TRUE) 
-  u_of_quadratic_utAu <- XtWy-.Rcpp_crossprod(Rzx, r_Zy, keep_names=FALSE,as_mat=TRUE)
-  if (TRUE) { # not clear why solve(cross_Rxx,.) would work and not chol() 
-    chol_XX <- chol(cross_Rxx)
-    r_Xy <- backsolve(chol_XX, u_of_quadratic_utAu, transpose=TRUE) ## I wrote "transpose since chol() provides a tcrossfac". ?? 
-    ryy2 <- sum(pwy_o*pwy_o) - sum(r_Zy@x*r_Zy@x) - sum(r_Xy*r_Xy)
+  XW <- sXaug_blocks$XW
+  if (ncol(XW)) { # there are fixed effects in X
+    ZtWX <- tZW %*% XW
+    Rzx <- solve(CHM_ZZ, solve(CHM_ZZ,ZtWX,system="P"), system="L") # (maybe) dense but dge... # solve(CHM_ZZ, ZtWX[perm,], system="L") # 
+    cross_Rxx <- .Rcpp_crossprod(XW,BB=NULL, keep_names=FALSE,as_mat=TRUE) -
+      .Rcpp_crossprod(Rzx,BB=NULL, keep_names=FALSE,as_mat=TRUE) 
+    XtWy <- .Rcpp_crossprod(XW, pwy_o, keep_names=FALSE,as_mat=TRUE)
+    u_of_quadratic_utAu <- XtWy-.Rcpp_crossprod(Rzx, r_Zy, keep_names=FALSE,as_mat=TRUE)
+    if (TRUE) { # not clear why solve(cross_Rxx,.) would work and not chol() 
+      chol_XX <- chol(cross_Rxx)
+      r_Xy <- backsolve(chol_XX, u_of_quadratic_utAu, transpose=TRUE) ## I wrote "transpose since chol() provides a tcrossfac". ?? 
+      ryy2 <- sum(pwy_o*pwy_o) - sum(r_Zy@x*r_Zy@x) - sum(r_Xy*r_Xy)
+      absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
+                             logdet_b=sum(log(abs(.diagfast(chol_XX)))), ryy2=ryy2)
+    } else if (FALSE) {
+      cross_X <- .crossprod(XW, as_sym=FALSE)
+      chX <- chol(cross_X)
+      # RzxL <- as(Rzx %*% solve(chX),"dgCMatrix") # block (1,2) using a previous solve(CHM_ZZ
+      # uplo <- .cbind_dgC_dgC( drop0(as(0*cross_Z,"dgCMatrix")), RzxL) # 1,1, 1,2
+      uplo <- as(Rzx %*% solve(chX),"dgCMatrix") # block (1,2) using a previous solve(CHM_ZZ
+      dimnames(uplo) <- list(NULL,NULL)
+      uplo@Dim[2L] <- uplo@Dim[2L]+ncol(CHM_ZZ) 
+      uplo@p <- c(rep(0L,ncol(CHM_ZZ)), uplo@p) # nice way of adding a LHS block of zeros. # 1:3, 1:2
+      uplo@Dim[1L] <- uplo@Dim[2L]+1L # nice way of adding rows of zeros. # 1:3, 1:2
+      #
+      LXtWy <- drop(backsolve(chX,XtWy,transpose=TRUE)) # block 2,3
+      ywy <- sum(pwy_o*pwy_o)
+      ycol <- c(drop(r_Zy),LXtWy,0)/sqrt(ywy)
+      dim(ycol) <- c(length(ycol),1L)
+      uplo <- .cbind_dgC_dgC(uplo, as(ycol,"dgCMatrix"))
+      CHM_full <- as(Cholesky(forceSymmetric(uplo),Imult=1,perm=FALSE),"sparseMatrix") # this is not efficient
+      # updating permuted cholesky would make extracting a lower triangle, as below, a dubious approach.
+      #
+      tltblock <- t(.get_dtC_lower_RHS(dtC_full=CHM_full, ncol_remove=ncol(CHM_ZZ)))
+      tltblock <- .remove_utri_last_rowcol(tltblock)
+      #
+      chol_XX <- tltblock %*% chX 
+      lastx <- tail(CHM_full@x,1)
+      ryy2 <- ywy*lastx*lastx
+      absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
+                             logdet_b=sum(log(.diagfast(chol_XX))), ryy2=ryy2)
+    } else if (use_crossr22 <- TRUE) { # a bit slower (even using .Rcpp_crossprod)
+      # No need for the complex Utri_chol computation here, as sum(r_Xy^2) is easy to compute without it.
+      # Another place where one can avoid it is also labelled 'use_crossr22' in .solve_crossr22()
+      sum_r_Ry_2 <- .crossprod(u_of_quadratic_utAu, solve(cross_Rxx, u_of_quadratic_utAu))
+      ryy2 <- sum(pwy_o^2) - sum(r_Zy^2) - sum_r_Ry_2
+      absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
+                             logdet_b=determinant(cross_Rxx)$modulus[1]/2, ryy2=ryy2)
+    } else {
+      chol_XX <- .Utri_chol_by_qr(cross_Rxx) # chol(cross_Rxx) # chol_XX matrix is quite small -> same algos as in .calc_r22()
+      # ## test-poly test-random-slope test-ranCoefs; 
+      # test-random-slope  is slower by .Rcpp_backsolve() but only because of more precise, but longer, outer optim in (ares <- ...)
+      # this better result is by accumulated effects on the optimization path rather than by functional improvement.
+      # also .Rcpp_backsolve() visibly increases range(get_predVar(twolambda)[1:5]-get_predVar(onelambda)[1:5]) 
+      r_Xy <- backsolve(chol_XX, u_of_quadratic_utAu, transpose=TRUE) ## transpose since chol() provides a tcrossfac 
+      # .crossprod(Rzx, r_Zy) appears to be .crossprod(ZtWX, solve(CHM_ZZ, ZtWy, system = "A")) 
+      # but we still need Rzx and r_Zy 
+      ryy2 <- sum(pwy_o*pwy_o) - sum(r_Zy@x*r_Zy@x) - sum(r_Xy*r_Xy)
+      absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
+                             logdet_b=sum(log(abs(.diagfast(chol_XX)))), ryy2=ryy2)
+    }
+  } else { # no fixed effects... 
+    ryy2 <- sum(pwy_o*pwy_o) - sum(r_Zy@x*r_Zy@x)
     absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
-                           logdet_b=sum(log(abs(.diagfast(chol_XX)))), ryy2=ryy2)
-  } else if (FALSE) {
-    cross_X <- .crossprod(XW, as_sym=FALSE)
-    chX <- chol(cross_X)
-    # RzxL <- as(Rzx %*% solve(chX),"dgCMatrix") # block (1,2) using a previous solve(CHM_ZZ
-    # uplo <- .cbind_dgC_dgC( drop0(as(0*cross_Z,"dgCMatrix")), RzxL) # 1,1, 1,2
-    uplo <- as(Rzx %*% solve(chX),"dgCMatrix") # block (1,2) using a previous solve(CHM_ZZ
-    dimnames(uplo) <- list(NULL,NULL)
-    uplo@Dim[2L] <- uplo@Dim[2L]+ncol(CHM_ZZ) 
-    uplo@p <- c(rep(0L,ncol(CHM_ZZ)), uplo@p) # nice way of adding a LHS block of zeros. # 1:3, 1:2
-    uplo@Dim[1L] <- uplo@Dim[2L]+1L # nice way of adding rows of zeros. # 1:3, 1:2
-    #
-    LXtWy <- drop(backsolve(chX,XtWy,transpose=TRUE)) # block 2,3
-    ywy <- sum(pwy_o*pwy_o)
-    ycol <- c(drop(r_Zy),LXtWy,0)/sqrt(ywy)
-    dim(ycol) <- c(length(ycol),1L)
-    uplo <- .cbind_dgC_dgC(uplo, as(ycol,"dgCMatrix"))
-    CHM_full <- as(Cholesky(forceSymmetric(uplo),Imult=1,perm=FALSE),"sparseMatrix") # this is not efficient
-    # updating permuted cholesky would make extracting a lower triangle, as below, a dubious approach.
-    #
-    tltblock <- t(.get_dtC_lower_RHS(dtC_full=CHM_full, ncol_remove=ncol(CHM_ZZ)))
-    tltblock <- .remove_utri_last_rowcol(tltblock)
-    #
-    chol_XX <- tltblock %*% chX 
-    lastx <- tail(CHM_full@x,1)
-    ryy2 <- ywy*lastx*lastx
-    absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
-                           logdet_b=sum(log(.diagfast(chol_XX))), ryy2=ryy2)
-  } else if (use_crossr22 <- TRUE) { # a bit slower (even using .Rcpp_crossprod)
-    # No need for the complex Utri_chol computation here, as sum(r_Xy^2) is easy to compute without it.
-    # Another place where one can avoid it is also labelled 'use_crossr22' in .solve_crossr22()
-    sum_r_Ry_2 <- .crossprod(u_of_quadratic_utAu, solve(cross_Rxx, u_of_quadratic_utAu))
-    ryy2 <- sum(pwy_o^2) - sum(r_Zy^2) - sum_r_Ry_2
-    absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
-                           logdet_b=determinant(cross_Rxx)$modulus[1]/2, ryy2=ryy2)
-  } else {
-    chol_XX <- .Utri_chol_by_qr(cross_Rxx) # chol(cross_Rxx) # chol_XX matrix is quite small -> same algos as in .calc_r22()
-    # ## test-poly test-random-slope test-ranCoefs; 
-    # test-random-slope  is slower by .Rcpp_backsolve() but only because of more precise, but longer, outer optim in (ares <- ...)
-    # this better result is by accumulated effects on the optimization path rather than by functional improvement.
-    # also .Rcpp_backsolve() visibly increases range(get_predVar(twolambda)[1:5]-get_predVar(onelambda)[1:5]) 
-    r_Xy <- backsolve(chol_XX, u_of_quadratic_utAu, transpose=TRUE) ## transpose since chol() provides a tcrossfac 
-    # .crossprod(Rzx, r_Zy) appears to be .crossprod(ZtWX, solve(CHM_ZZ, ZtWy, system = "A")) 
-    # but we still need Rzx and r_Zy 
-    ryy2 <- sum(pwy_o*pwy_o) - sum(r_Zy@x*r_Zy@x) - sum(r_Xy*r_Xy)
-    absdiagR_terms <- list(logdet_v=determinant(CHM_ZZ)$modulus[1], 
-                           logdet_b=sum(log(abs(.diagfast(chol_XX)))), ryy2=ryy2)
+                           logdet_b=0, ryy2=ryy2)
   }
   return(absdiagR_terms)
 }
@@ -718,7 +726,7 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
         logdet_R_scaled_b_v <- sum(log(absdiagR[-nc]))
         X_scaled_H_unscaled_logdet_r22 <- sum(log(absdiagR)[-c(seq(n_u_h),nc)]) -pforpv*log(H_global_scale)/2 
         ## -pforpv*log(H_global_scale)/2 for consistency with get_from_MME(sXaug,"logdet_r22") assuming the latter is correct
-      } else if (inherits(sXaug,"sXaug_blocks")) {
+      } else if (inherits(sXaug,"sXaug_blocks")) { # SPCORR !
         pwphi <- 1/(prior_weights) ## vector
         y_o <- drop(processed$y-processed$off)
         pwy_o <- y_o*sqrt(extranorm/pwphi)
@@ -741,9 +749,13 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
         y_o <- (processed$y-processed$off)
         pwy_o <- y_o*sqrt(extranorm/pwphi)
         if (inherits(sXaug,"dgCMatrix")) {
-          I00_ZXy <- .adhoc_cbind_dtC_sXaug_pxy_o(sXaug, pwy_o, n_u_h) ## distinctly faster
-        } else if (is.numeric(sXaug)) { ## not in routine tests
-          I00_ZXy <- .Rcpp_dense_cbind_mat_vec(sXaug, c(rep(0,n_u_h),pwy_o)) 
+          I00_ZXy <- .adhoc_cbind_dgC_sXaug_pxy_o(sXaug, pwy_o, n_u_h) ## distinctly faster
+        } else if (is.numeric(sXaug)) { ## not in routine tests but in CAR_timings
+          I00_ZXy <- .Rcpp_dense_cbind_mat_vec(sXaug, c(rep(0,n_u_h),pwy_o)) # typically costly ./.
+          # as a big matrix must be allocated each time .calc_APHLs_by_augZXy_or_sXaug) is called.
+          # This is where assignment in place in a stored template would be useful, but pure R will not avoid local copies. I tried
+          # I00_ZXy <- .update_I00_ZXy(sXaug, pwy_o, n_u_h)
+          # but this was slow.
         } else I00_ZXy <- cbind(sXaug,c(rep(0,n_u_h),pwy_o)) ## this cbind takes time...
         # Rcpp version of cbind for sparse matrices : https://stackoverflow.com/questions/45875668/rcpp-eigen-sparse-matrix-cbind#
         # but the gain is small...
@@ -930,10 +942,10 @@ get_from_MME_default.matrix <- function(sXaug,which="",szAug=NULL,B=NULL,...) {
 
 .dsCsum <- function(A, B, keep_names=FALSE) {
   if ( any(dim(A)!=dim(B))) stop("Dimensions of the two matrices are not identical") # if unprotected, causing hard crash
-  X <- .Rcpp_Csum(A,B)
-  X <- forceSymmetric(X)
-  if (keep_names) dimnames(X) <- dimnames(A)
-  return(X)
+  B <- .Rcpp_Csum(A,B)
+  B <- forceSymmetric(B)
+  if (keep_names) dimnames(B) <- dimnames(A)
+  return(B)
 }
 
 
