@@ -22,7 +22,7 @@
 
 .safe_opt <- function(init, objfn, lower, upper, verbose, maxeval_corr=.spaMM.data$options$maxeval_corr, 
                       recheck_at_bound=.spaMM.data$options$recheck_at_bound, 
-                      adjust_init=list(), 
+                      adjust_init=list(), # to constrain the initial value
                       LowUp,  
                       ...) { # minimization
   names_init <- names(init) # may be lost in later operations
@@ -84,5 +84,5 @@
     if (verbose>1L) print(c(objective=prevmin,next_init=init))
   }
   names(optr$solution) <- names_init
-  return(optr) # use nloptr format (solution, objective) for return
+  return(optr) # use nloptr format (solution, objective) for return, but $solution is named vector
 }
\ No newline at end of file
